/*
 * BEGIN LICENSE
 * This file is in the public domain
 * END LICENSE
 */
using Dee;
using Gee;
using Config;

/* Developer docs: */
/*                 */
/*   https://wiki.ubuntu.com/Unity/Lenses */
/*   http://developer.ubuntu.com/api/ubuntu-12.04/python/Unity-5.0.html */

namespace Unity.SampleLens {
  
  /* DBus name for the lens. Must match out .lens file */
  const string BUS_NAME = "com.canonical.Unity.SampleLens";

  /**
   * The Daemon class implements all of the logic for the lens.
   */
  public class Daemon : GLib.Object
  {
    private Unity.Lens lens;
    private Unity.Scope scope;

    construct
    {
      /* Set up the lens object (make sure it matches info in the .lens file */
      lens = new Unity.Lens ("/com/canonical/unity/samplelens", "sample");
      lens.search_hint = _("Search Stuff");
      lens.visible = true;
      lens.search_in_global = true;

      populate_categories ();
      populate_filters ();

      /* Create a scope so we can add results to the lens */
      scope = new Unity.Scope ("/com/canonical/unity/samplelens/samplescope");

      /* Let's ignore searches where only the leading or trailing whitespace
       * changes */
      scope.generate_search_key.connect ((lens_search) =>
      {
        return lens_search.search_string.strip ();
      });
      
      /* Listen for search changes */
      scope.search_changed.connect (on_search_changed);

      /* Re-do the search if the filters changed */
      scope.filters_changed.connect (() =>
      {
        message ("filters got updated...");
        scope.queue_search_changed (SearchType.DEFAULT);
      });

      /* Register the scope we created as local */
      lens.add_local_scope (scope);

      /* Export the lens on the bus. Unity can see us past this point */
      try {
        lens.export ();
      } catch (IOError error) {
        critical ("Failed to export DBus service for '%s': %s",
                  lens.dbus_path, error.message);
      }
    }

    private void populate_categories ()
    {
      GLib.List<Unity.Category> categories = new GLib.List<Unity.Category> ();

      /* The offsets should match those from the Category enum (schemas.vala) */
      var cat = new Unity.Category (_("All Stuff"),
                                    new ThemedIcon ("web-browser"));
      categories.append (cat);
      cat = new Unity.Category (_("My Stuff"),
                                new ThemedIcon ("web-browser"));
      categories.append (cat);
      cat = new Unity.Category (_("Your Stuff"),
                                new ThemedIcon ("web-browser"));
      categories.append (cat);

      lens.categories = categories;
    }

    private void populate_filters ()
    {
      GLib.List<Unity.Filter> filters = new GLib.List<Unity.Filter> ();

      var filter = new CheckOptionFilter ("filter1", _("Check filter"));
      filter.sort_type = Unity.OptionsFilter.SortType.DISPLAY_NAME;

      filter.add_option ("option1", _("Option 1"));
      filter.add_option ("option2", _("Option 2"));
      filter.add_option ("option3", _("Option 3"));
      filter.add_option ("option4", _("Option 4"));

      filters.append (filter);

      lens.filters = filters;
    }

    private void on_search_changed (Unity.Scope scope,
                                    Unity.LensSearch search,
                                    Unity.SearchType search_type,
                                    Cancellable cancellable)
    {
      /* We can either do the search here, or dispatch it into an async method
       * if we're going to contact a webservice or do an asynchronous action */
      start_search.begin (search, search_type, cancellable);
    }

    private async void start_search (Unity.LensSearch search,
                                     Unity.SearchType search_type,
                                     Cancellable cancellable)
    {
      message ("Started search for %s", search.search_string);
      
      /* Simulate that this search takes a while to get the results,
       * you could contact a web service or sweep the disc here etc. */
      Timeout.add (500, start_search.callback);
      yield;

      if (cancellable.is_cancelled ())
      {
        /* we got another search meanwhile, let's just end this one */
        return;
      }

      /* We have some results now, let's clear all the old ones and add
       * the new to the results */
      var model = search.results_model;
      model.clear ();

      var icon = ContentType.get_icon ("inode/directory");

      model.append ("file:///home/",     // URI of the result
                    icon.to_string (),   // string representation of an icon
                    Category.ALL_STUFF,  // category index
                    "inode/directory",   // mime-type of the URI,
                    "Home folder",       // display name of the result
                    "",                  // comment
                    "file:///home/");    // URI for drag and drop

      /* And one more */
      var uri = "http://www.google.com/?q=%s".printf (search.search_string);
      model.append (uri,
                    ContentType.get_icon ("text/html").to_string (),
                    Category.ALL_STUFF,
                    "text/html",
                    "\"%s\" on Google".printf (search.search_string),
                    "",
                    uri);

      search.finished ();
    }

  } /* End Daemon class */

} /* End Unity.SampleLens namespace */
